# Generated by ariadne-codegen
# Source: operations.graphql

from typing import List

from stigg._vendors.pydantic import Field

from .base_model import BaseModel
from .fragments import PageInfoFragment, SubscriptionQueryFragment


class Subscriptions(BaseModel):
    subscriptions: "SubscriptionsSubscriptions"


class SubscriptionsSubscriptions(BaseModel):
    edges: List["SubscriptionsSubscriptionsEdges"]
    page_info: "SubscriptionsSubscriptionsPageInfo" = Field(alias="pageInfo")


class SubscriptionsSubscriptionsEdges(BaseModel):
    node: "SubscriptionsSubscriptionsEdgesNode"


class SubscriptionsSubscriptionsEdgesNode(SubscriptionQueryFragment):
    pass


class SubscriptionsSubscriptionsPageInfo(PageInfoFragment):
    pass


Subscriptions.model_rebuild()
SubscriptionsSubscriptions.model_rebuild()
SubscriptionsSubscriptionsEdges.model_rebuild()



"""Validator functions for standard library types.

Import of this module is deferred since it contains imports of many standard library modules.
"""

from __future__ import annotations as _annotations

import math
import re
import typing
from ipaddress import IPv4Address, IPv4Interface, IPv4Network, IPv6Address, IPv6Interface, IPv6Network
from typing import Any

from pydantic_core import PydanticCustomError, core_schema
from pydantic_core._pydantic_core import PydanticKnownError


def sequence_validator(
    __input_value: typing.Sequence[Any],
    validator: core_schema.ValidatorFunctionWrapHandler,
) -> typing.Sequence[Any]:
    """Validator for `Sequence` types, isinstance(v, Sequence) has already been called."""
    value_type = type(__input_value)

    # We don't accept any plain string as a sequence
    # Relevant issue: https://github.com/pydantic/pydantic/issues/5595
    if issubclass(value_type, (str, bytes)):
        raise PydanticCustomError(
            'sequence_str',
            "'{type_name}' instances are not allowed as a Sequence value",
            {'type_name': value_type.__name__},
        )

    v_list = validator(__input_value)

    # the rest of the logic is just re-creating the original type from `v_list`
    if value_type == list:
        return v_list
    elif issubclass(value_type, range):
        # return the list as we probably can't re-create the range
        return v_list
    else:
        # best guess at how to re-create the original type, more custom construction logic might be required
        return value_type(v_list)  # type: ignore[call-arg]


def import_string(value: Any) -> Any:
    if isinstance(value, str):
        try:
            return _import_string_logic(value)
        except ImportError as e:
            raise PydanticCustomError('import_error', 'Invalid python path: {error}', {'error': str(e)}) from e
    else:
        # otherwise we just return the value and let the next validator do the rest of the work
        return value


def _import_string_logic(dotted_path: str) -> Any:
    """Inspired by uvicorn — dotted paths should include a colon before the final item if that item is not a module.
    (This is necessary to distinguish between a submodule and an attribute when there is a conflict.).

    If the dotted path does not include a colon and the final item is not a valid module, importing as an attribute
    rather than a submodule will be attempted automatically.

    So, for example, the following values of `dotted_path` result in the following returned values:
    * 'collections': <module 'collections'>
    * 'collections.abc': <module 'collections.abc'>
    * 'collections.abc:Mapping': <class 'collections.abc.Mapping'>
    * `collections.abc.Mapping`: <class 'collections.abc.Mapping'> (though this is a bit slower than the previous line)

    An error will be raised under any of the following scenarios:
    * `dotted_path` contains more than one colon (e.g., 'collections:abc:Mapping')
    * the substring of `dotted_path` before the colon is not a valid module in the environment (e.g., '123:Mapping')
    * the substring of `dotted_path` after the colon is not an attribute of the module (e.g., 'collections:abc123')
    """
    from importlib import import_module

    components = dotted_path.strip().split(':')
    if len(components) > 2:
        raise ImportError(f"Import strings should have at most one ':'; received {dotted_path!r}")

    module_path = components[0]
    if not module_path:
        raise ImportError(f'Import strings should have a nonempty module name; received {dotted_path!r}')

    try:
        module = import_module(module_path)
    except ModuleNotFoundError as e:
        if '.' in module_path:
            # Check if it would be valid if the final item was separated from its module with a `:`
            maybe_module_path, maybe_attribute = dotted_path.strip().rsplit('.', 1)
            try:
                return _import_string_logic(f'{maybe_module_path}:{maybe_attribute}')
            except ImportError:
                pass
            raise ImportError(f'No module named {module_path!r}') from e
        raise e

    if len(components) > 1:
        attribute = components[1]
        try:
            return getattr(module, attribute)
        except AttributeError as e:
            raise ImportError(f'cannot import name {attribute!r} from {module_path!r}') from e
    else:
        return module


def pattern_either_validator(__input_value: Any) -> typing.Pattern[Any]:
    if isinstance(__input_value, typing.Pattern):
        return __input_value
    elif isinstance(__input_value, (str, bytes)):
        # todo strict mode
        return compile_pattern(__input_value)  # type: ignore
    else:
        raise PydanticCustomError('pattern_type', 'Input should be a valid pattern')


def pattern_str_validator(__input_value: Any) -> typing.Pattern[str]:
    if isinstance(__input_value, typing.Pattern):
        if isinstance(__input_value.pattern, str):
            return __input_value
        else:
            raise PydanticCustomError('pattern_str_type', 'Input should be a string pattern')
    elif isinstance(__input_value, str):
        return compile_pattern(__input_value)
    elif isinstance(__input_value, bytes):
        raise PydanticCustomError('pattern_str_type', 'Input should be a string pattern')
    else:
        raise PydanticCustomError('pattern_type', 'Input should be a valid pattern')


def pattern_bytes_validator(__input_value: Any) -> typing.Pattern[bytes]:
    if isinstance(__input_value, typing.Pattern):
        if isinstance(__input_value.pattern, bytes):
            return __input_value
        else:
            raise PydanticCustomError('pattern_bytes_type', 'Input should be a bytes pattern')
    elif isinstance(__input_value, bytes):
        return compile_pattern(__input_value)
    elif isinstance(__input_value, str):
        raise PydanticCustomError('pattern_bytes_type', 'Input should be a bytes pattern')
    else:
        raise PydanticCustomError('pattern_type', 'Input should be a valid pattern')


PatternType = typing.TypeVar('PatternType', str, bytes)


def compile_pattern(pattern: PatternType) -> typing.Pattern[PatternType]:
    try:
        return re.compile(pattern)
    except re.error:
        raise PydanticCustomError('pattern_regex', 'Input should be a valid regular expression')


def ip_v4_address_validator(__input_value: Any) -> IPv4Address:
    if isinstance(__input_value, IPv4Address):
        return __input_value

    try:
        return IPv4Address(__input_value)
    except ValueError:
        raise PydanticCustomError('ip_v4_address', 'Input is not a valid IPv4 address')


def ip_v6_address_validator(__input_value: Any) -> IPv6Address:
    if isinstance(__input_value, IPv6Address):
        return __input_value

    try:
        return IPv6Address(__input_value)
    except ValueError:
        raise PydanticCustomError('ip_v6_address', 'Input is not a valid IPv6 address')


def ip_v4_network_validator(__input_value: Any) -> IPv4Network:
    """Assume IPv4Network initialised with a default `strict` argument.

    See more:
    https://docs.python.org/library/ipaddress.html#ipaddress.IPv4Network
    """
    if isinstance(__input_value, IPv4Network):
        return __input_value

    try:
        return IPv4Network(__input_value)
    except ValueError:
        raise PydanticCustomError('ip_v4_network', 'Input is not a valid IPv4 network')


def ip_v6_network_validator(__input_value: Any) -> IPv6Network:
    """Assume IPv6Network initialised with a default `strict` argument.

    See more:
    https://docs.python.org/library/ipaddress.html#ipaddress.IPv6Network
    """
    if isinstance(__input_value, IPv6Network):
        return __input_value

    try:
        return IPv6Network(__input_value)
    except ValueError:
        raise PydanticCustomError('ip_v6_network', 'Input is not a valid IPv6 network')


def ip_v4_interface_validator(__input_value: Any) -> IPv4Interface:
    if isinstance(__input_value, IPv4Interface):
        return __input_value

    try:
        return IPv4Interface(__input_value)
    except ValueError:
        raise PydanticCustomError('ip_v4_interface', 'Input is not a valid IPv4 interface')


def ip_v6_interface_validator(__input_value: Any) -> IPv6Interface:
    if isinstance(__input_value, IPv6Interface):
        return __input_value

    try:
        return IPv6Interface(__input_value)
    except ValueError:
        raise PydanticCustomError('ip_v6_interface', 'Input is not a valid IPv6 interface')


def greater_than_validator(x: Any, gt: Any) -> Any:
    if not (x > gt):
        raise PydanticKnownError('greater_than', {'gt': gt})
    return x


def greater_than_or_equal_validator(x: Any, ge: Any) -> Any:
    if not (x >= ge):
        raise PydanticKnownError('greater_than_equal', {'ge': ge})
    return x


def less_than_validator(x: Any, lt: Any) -> Any:
    if not (x < lt):
        raise PydanticKnownError('less_than', {'lt': lt})
    return x


def less_than_or_equal_validator(x: Any, le: Any) -> Any:
    if not (x <= le):
        raise PydanticKnownError('less_than_equal', {'le': le})
    return x


def multiple_of_validator(x: Any, multiple_of: Any) -> Any:
    if not (x % multiple_of == 0):
        raise PydanticKnownError('multiple_of', {'multiple_of': multiple_of})
    return x


def min_length_validator(x: Any, min_length: Any) -> Any:
    if not (len(x) >= min_length):
        raise PydanticKnownError(
            'too_short',
            {'field_type': 'Value', 'min_length': min_length, 'actual_length': len(x)},
        )
    return x


def max_length_validator(x: Any, max_length: Any) -> Any:
    if len(x) > max_length:
        raise PydanticKnownError(
            'too_long',
            {'field_type': 'Value', 'max_length': max_length, 'actual_length': len(x)},
        )
    return x


def forbid_inf_nan_check(x: Any) -> Any:
    if not math.isfinite(x):
        raise PydanticKnownError('finite_number')
    return x



"""Defining fields on models."""
from __future__ import annotations as _annotations

import dataclasses
import inspect
import typing
from copy import copy
from dataclasses import Field as DataclassField
from functools import cached_property
from typing import Any, ClassVar
from warnings import warn

import annotated_types
import typing_extensions
from pydantic_core import PydanticUndefined
from typing_extensions import Literal, Unpack

from . import types
from ._internal import _decorators, _fields, _generics, _internal_dataclass, _repr, _typing_extra, _utils
from .aliases import AliasChoices, AliasPath
from .config import JsonDict
from .errors import PydanticUserError
from .warnings import PydanticDeprecatedSince20

if typing.TYPE_CHECKING:
    from ._internal._repr import ReprArgs
else:
    # See PyCharm issues https://youtrack.jetbrains.com/issue/PY-21915
    # and https://youtrack.jetbrains.com/issue/PY-51428
    DeprecationWarning = PydanticDeprecatedSince20


_Unset: Any = PydanticUndefined


class _FromFieldInfoInputs(typing_extensions.TypedDict, total=False):
    """This class exists solely to add type checking for the `**kwargs` in `FieldInfo.from_field`."""

    annotation: type[Any] | None
    default_factory: typing.Callable[[], Any] | None
    alias: str | None
    alias_priority: int | None
    validation_alias: str | AliasPath | AliasChoices | None
    serialization_alias: str | None
    title: str | None
    description: str | None
    examples: list[Any] | None
    exclude: bool | None
    gt: float | None
    ge: float | None
    lt: float | None
    le: float | None
    multiple_of: float | None
    strict: bool | None
    min_length: int | None
    max_length: int | None
    pattern: str | None
    allow_inf_nan: bool | None
    max_digits: int | None
    decimal_places: int | None
    union_mode: Literal['smart', 'left_to_right'] | None
    discriminator: str | types.Discriminator | None
    json_schema_extra: JsonDict | typing.Callable[[JsonDict], None] | None
    frozen: bool | None
    validate_default: bool | None
    repr: bool
    init: bool | None
    init_var: bool | None
    kw_only: bool | None


class _FieldInfoInputs(_FromFieldInfoInputs, total=False):
    """This class exists solely to add type checking for the `**kwargs` in `FieldInfo.__init__`."""

    default: Any


class FieldInfo(_repr.Representation):
    """This class holds information about a field.

    `FieldInfo` is used for any field definition regardless of whether the [`Field()`][pydantic.fields.Field]
    function is explicitly used.

    !!! warning
        You generally shouldn't be creating `FieldInfo` directly, you'll only need to use it when accessing
        [`BaseModel`][pydantic.main.BaseModel] `.model_fields` internals.

    Attributes:
        annotation: The type annotation of the field.
        default: The default value of the field.
        default_factory: The factory function used to construct the default for the field.
        alias: The alias name of the field.
        alias_priority: The priority of the field's alias.
        validation_alias: The validation alias of the field.
        serialization_alias: The serialization alias of the field.
        title: The title of the field.
        description: The description of the field.
        examples: List of examples of the field.
        exclude: Whether to exclude the field from the model serialization.
        discriminator: Field name or Discriminator for discriminating the type in a tagged union.
        json_schema_extra: A dict or callable to provide extra JSON schema properties.
        frozen: Whether the field is frozen.
        validate_default: Whether to validate the default value of the field.
        repr: Whether to include the field in representation of the model.
        init: Whether the field should be included in the constructor of the dataclass.
        init_var: Whether the field should _only_ be included in the constructor of the dataclass, and not stored.
        kw_only: Whether the field should be a keyword-only argument in the constructor of the dataclass.
        metadata: List of metadata constraints.
    """

    annotation: type[Any] | None
    default: Any
    default_factory: typing.Callable[[], Any] | None
    alias: str | None
    alias_priority: int | None
    validation_alias: str | AliasPath | AliasChoices | None
    serialization_alias: str | None
    title: str | None
    description: str | None
    examples: list[Any] | None
    exclude: bool | None
    discriminator: str | types.Discriminator | None
    json_schema_extra: JsonDict | typing.Callable[[JsonDict], None] | None
    frozen: bool | None
    validate_default: bool | None
    repr: bool
    init: bool | None
    init_var: bool | None
    kw_only: bool | None
    metadata: list[Any]

    __slots__ = (
        'annotation',
        'default',
        'default_factory',
        'alias',
        'alias_priority',
        'validation_alias',
        'serialization_alias',
        'title',
        'description',
        'examples',
        'exclude',
        'discriminator',
        'json_schema_extra',
        'frozen',
        'validate_default',
        'repr',
        'init',
        'init_var',
        'kw_only',
        'metadata',
        '_attributes_set',
    )

    # used to convert kwargs to metadata/constraints,
    # None has a special meaning - these items are collected into a `PydanticGeneralMetadata`
    metadata_lookup: ClassVar[dict[str, typing.Callable[[Any], Any] | None]] = {
        'strict': types.Strict,
        'gt': annotated_types.Gt,
        'ge': annotated_types.Ge,
        'lt': annotated_types.Lt,
        'le': annotated_types.Le,
        'multiple_of': annotated_types.MultipleOf,
        'min_length': annotated_types.MinLen,
        'max_length': annotated_types.MaxLen,
        'pattern': None,
        'allow_inf_nan': None,
        'max_digits': None,
        'decimal_places': None,
        'union_mode': None,
    }

    def __init__(self, **kwargs: Unpack[_FieldInfoInputs]) -> None:
        """This class should generally not be initialized directly; instead, use the `pydantic.fields.Field` function
        or one of the constructor classmethods.

        See the signature of `pydantic.fields.Field` for more details about the expected arguments.
        """
        self._attributes_set = {k: v for k, v in kwargs.items() if v is not _Unset}
        kwargs = {k: _DefaultValues.get(k) if v is _Unset else v for k, v in kwargs.items()}  # type: ignore
        self.annotation, annotation_metadata = self._extract_metadata(kwargs.get('annotation'))

        default = kwargs.pop('default', PydanticUndefined)
        if default is Ellipsis:
            self.default = PydanticUndefined
        else:
            self.default = default

        self.default_factory = kwargs.pop('default_factory', None)

        if self.default is not PydanticUndefined and self.default_factory is not None:
            raise TypeError('cannot specify both default and default_factory')

        self.title = kwargs.pop('title', None)
        self.alias = kwargs.pop('alias', None)
        self.validation_alias = kwargs.pop('validation_alias', None)
        self.serialization_alias = kwargs.pop('serialization_alias', None)
        alias_is_set = any(alias is not None for alias in (self.alias, self.validation_alias, self.serialization_alias))
        self.alias_priority = kwargs.pop('alias_priority', None) or 2 if alias_is_set else None
        self.description = kwargs.pop('description', None)
        self.examples = kwargs.pop('examples', None)
        self.exclude = kwargs.pop('exclude', None)
        self.discriminator = kwargs.pop('discriminator', None)
        self.repr = kwargs.pop('repr', True)
        self.json_schema_extra = kwargs.pop('json_schema_extra', None)
        self.validate_default = kwargs.pop('validate_default', None)
        self.frozen = kwargs.pop('frozen', None)
        # currently only used on dataclasses
        self.init = kwargs.pop('init', None)
        self.init_var = kwargs.pop('init_var', None)
        self.kw_only = kwargs.pop('kw_only', None)

        self.metadata = self._collect_metadata(kwargs) + annotation_metadata  # type: ignore

    @staticmethod
    def from_field(default: Any = PydanticUndefined, **kwargs: Unpack[_FromFieldInfoInputs]) -> FieldInfo:
        """Create a new `FieldInfo` object with the `Field` function.

        Args:
            default: The default value for the field. Defaults to Undefined.
            **kwargs: Additional arguments dictionary.

        Raises:
            TypeError: If 'annotation' is passed as a keyword argument.

        Returns:
            A new FieldInfo object with the given parameters.

        Example:
            This is how you can create a field with default value like this:

            ```python
            import pydantic

            class MyModel(pydantic.BaseModel):
                foo: int = pydantic.Field(4)
            ```
        """
        if 'annotation' in kwargs:
            raise TypeError('"annotation" is not permitted as a Field keyword argument')
        return FieldInfo(default=default, **kwargs)

    @staticmethod
    def from_annotation(annotation: type[Any]) -> FieldInfo:
        """Creates a `FieldInfo` instance from a bare annotation.

        This function is used internally to create a `FieldInfo` from a bare annotation like this:

        ```python
        import pydantic

        class MyModel(pydantic.BaseModel):
            foo: int  # <-- like this
        ```

        We also account for the case where the annotation can be an instance of `Annotated` and where
        one of the (not first) arguments in `Annotated` is an instance of `FieldInfo`, e.g.:

        ```python
        import annotated_types
        from typing_extensions import Annotated

        import pydantic

        class MyModel(pydantic.BaseModel):
            foo: Annotated[int, annotated_types.Gt(42)]
            bar: Annotated[int, pydantic.Field(gt=42)]
        ```

        Args:
            annotation: An annotation object.

        Returns:
            An instance of the field metadata.
        """
        final = False
        if _typing_extra.is_finalvar(annotation):
            final = True
            if annotation is not typing_extensions.Final:
                annotation = typing_extensions.get_args(annotation)[0]

        if _typing_extra.is_annotated(annotation):
            first_arg, *extra_args = typing_extensions.get_args(annotation)
            if _typing_extra.is_finalvar(first_arg):
                final = True
            field_info_annotations = [a for a in extra_args if isinstance(a, FieldInfo)]
            field_info = FieldInfo.merge_field_infos(*field_info_annotations, annotation=first_arg)
            if field_info:
                new_field_info = copy(field_info)
                new_field_info.annotation = first_arg
                new_field_info.frozen = final or field_info.frozen
                metadata: list[Any] = []
                for a in extra_args:
                    if not isinstance(a, FieldInfo):
                        metadata.append(a)
                    else:
                        metadata.extend(a.metadata)
                new_field_info.metadata = metadata
                return new_field_info

        return FieldInfo(annotation=annotation, frozen=final or None)

    @staticmethod
    def from_annotated_attribute(annotation: type[Any], default: Any) -> FieldInfo:
        """Create `FieldInfo` from an annotation with a default value.

        This is used in cases like the following:

        ```python
        import annotated_types
        from typing_extensions import Annotated

        import pydantic

        class MyModel(pydantic.BaseModel):
            foo: int = 4  # <-- like this
            bar: Annotated[int, annotated_types.Gt(4)] = 4  # <-- or this
            spam: Annotated[int, pydantic.Field(gt=4)] = 4  # <-- or this
        ```

        Args:
            annotation: The type annotation of the field.
            default: The default value of the field.

        Returns:
            A field object with the passed values.
        """
        if annotation is default:
            raise PydanticUserError(
                'Error when building FieldInfo from annotated attribute. '
                "Make sure you don't have any field name clashing with a type annotation ",
                code='unevaluable-type-annotation',
            )

        final = False
        if _typing_extra.is_finalvar(annotation):
            final = True
            if annotation is not typing_extensions.Final:
                annotation = typing_extensions.get_args(annotation)[0]

        if isinstance(default, FieldInfo):
            default.annotation, annotation_metadata = FieldInfo._extract_metadata(annotation)
            default.metadata += annotation_metadata
            default = default.merge_field_infos(
                *[x for x in annotation_metadata if isinstance(x, FieldInfo)], default, annotation=default.annotation
            )
            default.frozen = final or default.frozen
            return default
        elif isinstance(default, dataclasses.Field):
            init_var = False
            if annotation is dataclasses.InitVar:
                init_var = True
                annotation = Any
            elif isinstance(annotation, dataclasses.InitVar):
                init_var = True
                annotation = annotation.type
            pydantic_field = FieldInfo._from_dataclass_field(default)
            pydantic_field.annotation, annotation_metadata = FieldInfo._extract_metadata(annotation)
            pydantic_field.metadata += annotation_metadata
            pydantic_field = pydantic_field.merge_field_infos(
                *[x for x in annotation_metadata if isinstance(x, FieldInfo)],
                pydantic_field,
                annotation=pydantic_field.annotation,
            )
            pydantic_field.frozen = final or pydantic_field.frozen
            pydantic_field.init_var = init_var
            pydantic_field.init = getattr(default, 'init', None)
            pydantic_field.kw_only = getattr(default, 'kw_only', None)
            return pydantic_field
        else:
            if _typing_extra.is_annotated(annotation):
                first_arg, *extra_args = typing_extensions.get_args(annotation)
                field_infos = [a for a in extra_args if isinstance(a, FieldInfo)]
                field_info = FieldInfo.merge_field_infos(*field_infos, annotation=first_arg, default=default)
                metadata: list[Any] = []
                for a in extra_args:
                    if not isinstance(a, FieldInfo):
                        metadata.append(a)
                    else:
                        metadata.extend(a.metadata)
                field_info.metadata = metadata
                return field_info

            return FieldInfo(annotation=annotation, default=default, frozen=final or None)

    @staticmethod
    def merge_field_infos(*field_infos: FieldInfo, **overrides: Any) -> FieldInfo:
        """Merge `FieldInfo` instances keeping only explicitly set attributes.

        Later `FieldInfo` instances override earlier ones.

        Returns:
            FieldInfo: A merged FieldInfo instance.
        """
        flattened_field_infos: list[FieldInfo] = []
        for field_info in field_infos:
            flattened_field_infos.extend(x for x in field_info.metadata if isinstance(x, FieldInfo))
            flattened_field_infos.append(field_info)
        field_infos = tuple(flattened_field_infos)
        if len(field_infos) == 1:
            # No merging necessary, but we still need to make a copy and apply the overrides
            field_info = copy(field_infos[0])
            field_info._attributes_set.update(overrides)
            for k, v in overrides.items():
                setattr(field_info, k, v)
            return field_info  # type: ignore

        new_kwargs: dict[str, Any] = {}
        metadata = {}
        for field_info in field_infos:
            new_kwargs.update(field_info._attributes_set)
            for x in field_info.metadata:
                if not isinstance(x, FieldInfo):
                    metadata[type(x)] = x
        new_kwargs.update(overrides)
        field_info = FieldInfo(**new_kwargs)
        field_info.metadata = list(metadata.values())
        return field_info

    @staticmethod
    def _from_dataclass_field(dc_field: DataclassField[Any]) -> FieldInfo:
        """Return a new `FieldInfo` instance from a `dataclasses.Field` instance.

        Args:
            dc_field: The `dataclasses.Field` instance to convert.

        Returns:
            The corresponding `FieldInfo` instance.

        Raises:
            TypeError: If any of the `FieldInfo` kwargs does not match the `dataclass.Field` kwargs.
        """
        default = dc_field.default
        if default is dataclasses.MISSING:
            default = PydanticUndefined

        if dc_field.default_factory is dataclasses.MISSING:
            default_factory: typing.Callable[[], Any] | None = None
        else:
            default_factory = dc_field.default_factory

        # use the `Field` function so in correct kwargs raise the correct `TypeError`
        dc_field_metadata = {k: v for k, v in dc_field.metadata.items() if k in _FIELD_ARG_NAMES}
        return Field(default=default, default_factory=default_factory, repr=dc_field.repr, **dc_field_metadata)

    @staticmethod
    def _extract_metadata(annotation: type[Any] | None) -> tuple[type[Any] | None, list[Any]]:
        """Tries to extract metadata/constraints from an annotation if it uses `Annotated`.

        Args:
            annotation: The type hint annotation for which metadata has to be extracted.

        Returns:
            A tuple containing the extracted metadata type and the list of extra arguments.
        """
        if annotation is not None:
            if _typing_extra.is_annotated(annotation):
                first_arg, *extra_args = typing_extensions.get_args(annotation)
                return first_arg, list(extra_args)

        return annotation, []

    @staticmethod
    def _collect_metadata(kwargs: dict[str, Any]) -> list[Any]:
        """Collect annotations from kwargs.

        Args:
            kwargs: Keyword arguments passed to the function.

        Returns:
            A list of metadata objects - a combination of `annotated_types.BaseMetadata` and
                `PydanticMetadata`.
        """
        metadata: list[Any] = []
        general_metadata = {}
        for key, value in list(kwargs.items()):
            try:
                marker = FieldInfo.metadata_lookup[key]
            except KeyError:
                continue

            del kwargs[key]
            if value is not None:
                if marker is None:
                    general_metadata[key] = value
                else:
                    metadata.append(marker(value))
        if general_metadata:
            metadata.append(_fields.pydantic_general_metadata(**general_metadata))
        return metadata

    def get_default(self, *, call_default_factory: bool = False) -> Any:
        """Get the default value.

        We expose an option for whether to call the default_factory (if present), as calling it may
        result in side effects that we want to avoid. However, there are times when it really should
        be called (namely, when instantiating a model via `model_construct`).

        Args:
            call_default_factory: Whether to call the default_factory or not. Defaults to `False`.

        Returns:
            The default value, calling the default factory if requested or `None` if not set.
        """
        if self.default_factory is None:
            return _utils.smart_deepcopy(self.default)
        elif call_default_factory:
            return self.default_factory()
        else:
            return None

    def is_required(self) -> bool:
        """Check if the field is required (i.e., does not have a default value or factory).

        Returns:
            `True` if the field is required, `False` otherwise.
        """
        return self.default is PydanticUndefined and self.default_factory is None

    def rebuild_annotation(self) -> Any:
        """Attempts to rebuild the original annotation for use in function signatures.

        If metadata is present, it adds it to the original annotation using
        `Annotated`. Otherwise, it returns the original annotation as-is.

        Note that because the metadata has been flattened, the original annotation
        may not be reconstructed exactly as originally provided, e.g. if the original
        type had unrecognized annotations, or was annotated with a call to `pydantic.Field`.

        Returns:
            The rebuilt annotation.
        """
        if not self.metadata:
            return self.annotation
        else:
            # Annotated arguments must be a tuple
            return typing_extensions.Annotated[(self.annotation, *self.metadata)]  # type: ignore

    def apply_typevars_map(self, typevars_map: dict[Any, Any] | None, types_namespace: dict[str, Any] | None) -> None:
        """Apply a `typevars_map` to the annotation.

        This method is used when analyzing parametrized generic types to replace typevars with their concrete types.

        This method applies the `typevars_map` to the annotation in place.

        Args:
            typevars_map: A dictionary mapping type variables to their concrete types.
            types_namespace (dict | None): A dictionary containing related types to the annotated type.

        See Also:
            pydantic._internal._generics.replace_types is used for replacing the typevars with
                their concrete types.
        """
        annotation = _typing_extra.eval_type_lenient(self.annotation, types_namespace)
        self.annotation = _generics.replace_types(annotation, typevars_map)

    def __repr_args__(self) -> ReprArgs:
        yield 'annotation', _repr.PlainRepr(_repr.display_as_type(self.annotation))
        yield 'required', self.is_required()

        for s in self.__slots__:
            if s == '_attributes_set':
                continue
            if s == 'annotation':
                continue
            elif s == 'metadata' and not self.metadata:
                continue
            elif s == 'repr' and self.repr is True:
                continue
            if s == 'frozen' and self.frozen is False:
                continue
            if s == 'validation_alias' and self.validation_alias == self.alias:
                continue
            if s == 'serialization_alias' and self.serialization_alias == self.alias:
                continue
            if s == 'default_factory' and self.default_factory is not None:
                yield 'default_factory', _repr.PlainRepr(_repr.display_as_type(self.default_factory))
            else:
                value = getattr(self, s)
                if value is not None and value is not PydanticUndefined:
                    yield s, value


class _EmptyKwargs(typing_extensions.TypedDict):
    """This class exists solely to ensure that type checking warns about passing `**extra` in `Field`."""


_DefaultValues = dict(
    default=...,
    default_factory=None,
    alias=None,
    alias_priority=None,
    validation_alias=None,
    serialization_alias=None,
    title=None,
    description=None,
    examples=None,
    exclude=None,
    discriminator=None,
    json_schema_extra=None,
    frozen=None,
    validate_default=None,
    repr=True,
    init=None,
    init_var=None,
    kw_only=None,
    pattern=None,
    strict=None,
    gt=None,
    ge=None,
    lt=None,
    le=None,
    multiple_of=None,
    allow_inf_nan=None,
    max_digits=None,
    decimal_places=None,
    min_length=None,
    max_length=None,
)


# see: https://docs.pydantic.dev/latest/migration/#required-optional-and-nullable-fields
def Field(  # noqa: C901
    default: Any = None, # this hack to fix pydantic v2 migration, related to SCS-242/STIGG-2138
    *,
    default_factory: typing.Callable[[], Any] | None = _Unset,
    alias: str | None = _Unset,
    alias_priority: int | None = _Unset,
    validation_alias: str | AliasPath | AliasChoices | None = _Unset,
    serialization_alias: str | None = _Unset,
    title: str | None = _Unset,
    description: str | None = _Unset,
    examples: list[Any] | None = _Unset,
    exclude: bool | None = _Unset,
    discriminator: str | types.Discriminator | None = _Unset,
    json_schema_extra: JsonDict | typing.Callable[[JsonDict], None] | None = _Unset,
    frozen: bool | None = _Unset,
    validate_default: bool | None = _Unset,
    repr: bool = _Unset,
    init: bool | None = _Unset,
    init_var: bool | None = _Unset,
    kw_only: bool | None = _Unset,
    pattern: str | None = _Unset,
    strict: bool | None = _Unset,
    gt: float | None = _Unset,
    ge: float | None = _Unset,
    lt: float | None = _Unset,
    le: float | None = _Unset,
    multiple_of: float | None = _Unset,
    allow_inf_nan: bool | None = _Unset,
    max_digits: int | None = _Unset,
    decimal_places: int | None = _Unset,
    min_length: int | None = _Unset,
    max_length: int | None = _Unset,
    union_mode: Literal['smart', 'left_to_right'] = _Unset,
    **extra: Unpack[_EmptyKwargs],
) -> Any:
    """Usage docs: https://docs.pydantic.dev/2.6/concepts/fields

    Create a field for objects that can be configured.

    Used to provide extra information about a field, either for the model schema or complex validation. Some arguments
    apply only to number fields (`int`, `float`, `Decimal`) and some apply only to `str`.

    Note:
        - Any `_Unset` objects will be replaced by the corresponding value defined in the `_DefaultValues` dictionary. If a key for the `_Unset` object is not found in the `_DefaultValues` dictionary, it will default to `None`

    Args:
        default: Default value if the field is not set.
        default_factory: A callable to generate the default value, such as :func:`~datetime.utcnow`.
        alias: The name to use for the attribute when validating or serializing by alias.
            This is often used for things like converting between snake and camel case.
        alias_priority: Priority of the alias. This affects whether an alias generator is used.
        validation_alias: Like `alias`, but only affects validation, not serialization.
        serialization_alias: Like `alias`, but only affects serialization, not validation.
        title: Human-readable title.
        description: Human-readable description.
        examples: Example values for this field.
        exclude: Whether to exclude the field from the model serialization.
        discriminator: Field name or Discriminator for discriminating the type in a tagged union.
        json_schema_extra: A dict or callable to provide extra JSON schema properties.
        frozen: Whether the field is frozen. If true, attempts to change the value on an instance will raise an error.
        validate_default: If `True`, apply validation to the default value every time you create an instance.
            Otherwise, for performance reasons, the default value of the field is trusted and not validated.
        repr: A boolean indicating whether to include the field in the `__repr__` output.
        init: Whether the field should be included in the constructor of the dataclass.
            (Only applies to dataclasses.)
        init_var: Whether the field should _only_ be included in the constructor of the dataclass.
            (Only applies to dataclasses.)
        kw_only: Whether the field should be a keyword-only argument in the constructor of the dataclass.
            (Only applies to dataclasses.)
        strict: If `True`, strict validation is applied to the field.
            See [Strict Mode](../concepts/strict_mode.md) for details.
        gt: Greater than. If set, value must be greater than this. Only applicable to numbers.
        ge: Greater than or equal. If set, value must be greater than or equal to this. Only applicable to numbers.
        lt: Less than. If set, value must be less than this. Only applicable to numbers.
        le: Less than or equal. If set, value must be less than or equal to this. Only applicable to numbers.
        multiple_of: Value must be a multiple of this. Only applicable to numbers.
        min_length: Minimum length for strings.
        max_length: Maximum length for strings.
        pattern: Pattern for strings (a regular expression).
        allow_inf_nan: Allow `inf`, `-inf`, `nan`. Only applicable to numbers.
        max_digits: Maximum number of allow digits for strings.
        decimal_places: Maximum number of decimal places allowed for numbers.
        union_mode: The strategy to apply when validating a union. Can be `smart` (the default), or `left_to_right`.
            See [Union Mode](standard_library_types.md#union-mode) for details.
        extra: (Deprecated) Extra fields that will be included in the JSON schema.

            !!! warning Deprecated
                The `extra` kwargs is deprecated. Use `json_schema_extra` instead.

    Returns:
        A new [`FieldInfo`][pydantic.fields.FieldInfo]. The return annotation is `Any` so `Field` can be used on
            type-annotated fields without causing a type error.
    """
    # Check deprecated and removed params from V1. This logic should eventually be removed.
    const = extra.pop('const', None)  # type: ignore
    if const is not None:
        raise PydanticUserError('`const` is removed, use `Literal` instead', code='removed-kwargs')

    min_items = extra.pop('min_items', None)  # type: ignore
    if min_items is not None:
        warn('`min_items` is deprecated and will be removed, use `min_length` instead', DeprecationWarning)
        if min_length in (None, _Unset):
            min_length = min_items  # type: ignore

    max_items = extra.pop('max_items', None)  # type: ignore
    if max_items is not None:
        warn('`max_items` is deprecated and will be removed, use `max_length` instead', DeprecationWarning)
        if max_length in (None, _Unset):
            max_length = max_items  # type: ignore

    unique_items = extra.pop('unique_items', None)  # type: ignore
    if unique_items is not None:
        raise PydanticUserError(
            (
                '`unique_items` is removed, use `Set` instead'
                '(this feature is discussed in https://github.com/pydantic/pydantic-core/issues/296)'
            ),
            code='removed-kwargs',
        )

    allow_mutation = extra.pop('allow_mutation', None)  # type: ignore
    if allow_mutation is not None:
        warn('`allow_mutation` is deprecated and will be removed. use `frozen` instead', DeprecationWarning)
        if allow_mutation is False:
            frozen = True

    regex = extra.pop('regex', None)  # type: ignore
    if regex is not None:
        raise PydanticUserError('`regex` is removed. use `pattern` instead', code='removed-kwargs')

    if extra:
        warn(
            'Using extra keyword arguments on `Field` is deprecated and will be removed.'
            ' Use `json_schema_extra` instead.'
            f' (Extra keys: {", ".join(k.__repr__() for k in extra.keys())})',
            DeprecationWarning,
        )
        if not json_schema_extra or json_schema_extra is _Unset:
            json_schema_extra = extra  # type: ignore

    if (
        validation_alias
        and validation_alias is not _Unset
        and not isinstance(validation_alias, (str, AliasChoices, AliasPath))
    ):
        raise TypeError('Invalid `validation_alias` type. it should be `str`, `AliasChoices`, or `AliasPath`')

    if serialization_alias in (_Unset, None) and isinstance(alias, str):
        serialization_alias = alias

    if validation_alias in (_Unset, None):
        validation_alias = alias

    include = extra.pop('include', None)  # type: ignore
    if include is not None:
        warn('`include` is deprecated and does nothing. It will be removed, use `exclude` instead', DeprecationWarning)

    return FieldInfo.from_field(
        default,
        default_factory=default_factory,
        alias=alias,
        alias_priority=alias_priority,
        validation_alias=validation_alias,
        serialization_alias=serialization_alias,
        title=title,
        description=description,
        examples=examples,
        exclude=exclude,
        discriminator=discriminator,
        json_schema_extra=json_schema_extra,
        frozen=frozen,
        pattern=pattern,
        validate_default=validate_default,
        repr=repr,
        init=init,
        init_var=init_var,
        kw_only=kw_only,
        strict=strict,
        gt=gt,
        ge=ge,
        lt=lt,
        le=le,
        multiple_of=multiple_of,
        min_length=min_length,
        max_length=max_length,
        allow_inf_nan=allow_inf_nan,
        max_digits=max_digits,
        decimal_places=decimal_places,
        union_mode=union_mode,
    )


_FIELD_ARG_NAMES = set(inspect.signature(Field).parameters)
_FIELD_ARG_NAMES.remove('extra')  # do not include the varkwargs parameter


class ModelPrivateAttr(_repr.Representation):
    """A descriptor for private attributes in class models.

    !!! warning
        You generally shouldn't be creating `ModelPrivateAttr` instances directly, instead use
        `pydantic.fields.PrivateAttr`. (This is similar to `FieldInfo` vs. `Field`.)

    Attributes:
        default: The default value of the attribute if not provided.
        default_factory: A callable function that generates the default value of the
            attribute if not provided.
    """

    __slots__ = 'default', 'default_factory'

    def __init__(
        self, default: Any = PydanticUndefined, *, default_factory: typing.Callable[[], Any] | None = None
    ) -> None:
        self.default = default
        self.default_factory = default_factory

    if not typing.TYPE_CHECKING:
        # We put `__getattr__` in a non-TYPE_CHECKING block because otherwise, mypy allows arbitrary attribute access

        def __getattr__(self, item: str) -> Any:
            """This function improves compatibility with custom descriptors by ensuring delegation happens
            as expected when the default value of a private attribute is a descriptor.
            """
            if item in {'__get__', '__set__', '__delete__'}:
                if hasattr(self.default, item):
                    return getattr(self.default, item)
            raise AttributeError(f'{type(self).__name__!r} object has no attribute {item!r}')

    def __set_name__(self, cls: type[Any], name: str) -> None:
        """Preserve `__set_name__` protocol defined in https://peps.python.org/pep-0487."""
        if self.default is PydanticUndefined:
            return
        if not hasattr(self.default, '__set_name__'):
            return
        set_name = self.default.__set_name__
        if callable(set_name):
            set_name(cls, name)

    def get_default(self) -> Any:
        """Retrieve the default value of the object.

        If `self.default_factory` is `None`, the method will return a deep copy of the `self.default` object.

        If `self.default_factory` is not `None`, it will call `self.default_factory` and return the value returned.

        Returns:
            The default value of the object.
        """
        return _utils.smart_deepcopy(self.default) if self.default_factory is None else self.default_factory()

    def __eq__(self, other: Any) -> bool:
        return isinstance(other, self.__class__) and (self.default, self.default_factory) == (
            other.default,
            other.default_factory,
        )


def PrivateAttr(
    default: Any = PydanticUndefined,
    *,
    default_factory: typing.Callable[[], Any] | None = None,
) -> Any:
    """Usage docs: https://docs.pydantic.dev/2.6/concepts/models/#private-model-attributes

    Indicates that an attribute is intended for private use and not handled during normal validation/serialization.

    Private attributes are not validated by Pydantic, so it's up to you to ensure they are used in a type-safe manner.

    Private attributes are stored in `__private_attributes__` on the model.

    Args:
        default: The attribute's default value. Defaults to Undefined.
        default_factory: Callable that will be
            called when a default value is needed for this attribute.
            If both `default` and `default_factory` are set, an error will be raised.

    Returns:
        An instance of [`ModelPrivateAttr`][pydantic.fields.ModelPrivateAttr] class.

    Raises:
        ValueError: If both `default` and `default_factory` are set.
    """
    if default is not PydanticUndefined and default_factory is not None:
        raise TypeError('cannot specify both default and default_factory')

    return ModelPrivateAttr(
        default,
        default_factory=default_factory,
    )


@dataclasses.dataclass(**_internal_dataclass.slots_true)
class ComputedFieldInfo:
    """A container for data from `@computed_field` so that we can access it while building the pydantic-core schema.

    Attributes:
        decorator_repr: A class variable representing the decorator string, '@computed_field'.
        wrapped_property: The wrapped computed field property.
        return_type: The type of the computed field property's return value.
        alias: The alias of the property to be used during serialization.
        alias_priority: The priority of the alias. This affects whether an alias generator is used.
        title: Title of the computed field to include in the serialization JSON schema.
        description: Description of the computed field to include in the serialization JSON schema.
        examples: Example values of the computed field to include in the serialization JSON schema.
        json_schema_extra: A dict or callable to provide extra JSON schema properties.
        repr: A boolean indicating whether to include the field in the __repr__ output.
    """

    decorator_repr: ClassVar[str] = '@computed_field'
    wrapped_property: property
    return_type: Any
    alias: str | None
    alias_priority: int | None
    title: str | None
    description: str | None
    examples: list[Any] | None
    json_schema_extra: JsonDict | typing.Callable[[JsonDict], None] | None
    repr: bool


def _wrapped_property_is_private(property_: cached_property | property) -> bool:  # type: ignore
    """Returns true if provided property is private, False otherwise."""
    wrapped_name: str = ''

    if isinstance(property_, property):
        wrapped_name = getattr(property_.fget, '__name__', '')
    elif isinstance(property_, cached_property):  # type: ignore
        wrapped_name = getattr(property_.func, '__name__', '')  # type: ignore

    return wrapped_name.startswith('_') and not wrapped_name.startswith('__')


# this should really be `property[T], cached_property[T]` but property is not generic unlike cached_property
# See https://github.com/python/typing/issues/985 and linked issues
PropertyT = typing.TypeVar('PropertyT')


@typing.overload
def computed_field(
    *,
    alias: str | None = None,
    alias_priority: int | None = None,
    title: str | None = None,
    description: str | None = None,
    examples: list[Any] | None = None,
    json_schema_extra: JsonDict | typing.Callable[[JsonDict], None] | None = None,
    repr: bool = True,
    return_type: Any = PydanticUndefined,
) -> typing.Callable[[PropertyT], PropertyT]:
    ...


@typing.overload
def computed_field(__func: PropertyT) -> PropertyT:
    ...


def computed_field(
    __f: PropertyT | None = None,
    *,
    alias: str | None = None,
    alias_priority: int | None = None,
    title: str | None = None,
    description: str | None = None,
    examples: list[Any] | None = None,
    json_schema_extra: JsonDict | typing.Callable[[JsonDict], None] | None = None,
    repr: bool | None = None,
    return_type: Any = PydanticUndefined,
) -> PropertyT | typing.Callable[[PropertyT], PropertyT]:
    """Usage docs: https://docs.pydantic.dev/2.6/concepts/fields#the-computed_field-decorator

    Decorator to include `property` and `cached_property` when serializing models or dataclasses.

    This is useful for fields that are computed from other fields, or for fields that are expensive to compute and should be cached.

    ```py
    from pydantic import BaseModel, computed_field

    class Rectangle(BaseModel):
        width: int
        length: int

        @computed_field
        @property
        def area(self) -> int:
            return self.width * self.length

    print(Rectangle(width=3, length=2).model_dump())
    #> {'width': 3, 'length': 2, 'area': 6}
    ```

    If applied to functions not yet decorated with `@property` or `@cached_property`, the function is
    automatically wrapped with `property`. Although this is more concise, you will lose IntelliSense in your IDE,
    and confuse static type checkers, thus explicit use of `@property` is recommended.

    !!! warning "Mypy Warning"
        Even with the `@property` or `@cached_property` applied to your function before `@computed_field`,
        mypy may throw a `Decorated property not supported` error.
        See [mypy issue #1362](https://github.com/python/mypy/issues/1362), for more information.
        To avoid this error message, add `# type: ignore[misc]` to the `@computed_field` line.

        [pyright](https://github.com/microsoft/pyright) supports `@computed_field` without error.

    ```py
    import random

    from pydantic import BaseModel, computed_field

    class Square(BaseModel):
        width: float

        @computed_field
        def area(self) -> float:  # converted to a `property` by `computed_field`
            return round(self.width**2, 2)

        @area.setter
        def area(self, new_area: float) -> None:
            self.width = new_area**0.5

        @computed_field(alias='the magic number', repr=False)
        def random_number(self) -> int:
            return random.randint(0, 1_000)

    square = Square(width=1.3)

    # `random_number` does not appear in representation
    print(repr(square))
    #> Square(width=1.3, area=1.69)

    print(square.random_number)
    #> 3

    square.area = 4

    print(square.model_dump_json(by_alias=True))
    #> {"width":2.0,"area":4.0,"the magic number":3}
    ```

    !!! warning "Overriding with `computed_field`"
        You can't override a field from a parent class with a `computed_field` in the child class.
        `mypy` complains about this behavior if allowed, and `dataclasses` doesn't allow this pattern either.
        See the example below:

    ```py
    from pydantic import BaseModel, computed_field

    class Parent(BaseModel):
        a: str

    try:

        class Child(Parent):
            @computed_field
            @property
            def a(self) -> str:
                return 'new a'

    except ValueError as e:
        print(repr(e))
        #> ValueError("you can't override a field with a computed field")
    ```

    Private properties decorated with `@computed_field` have `repr=False` by default.

    ```py
    from functools import cached_property

    from pydantic import BaseModel, computed_field

    class Model(BaseModel):
        foo: int

        @computed_field
        @cached_property
        def _private_cached_property(self) -> int:
            return -self.foo

        @computed_field
        @property
        def _private_property(self) -> int:
            return -self.foo

    m = Model(foo=1)
    print(repr(m))
    #> M(foo=1)
    ```

    Args:
        __f: the function to wrap.
        alias: alias to use when serializing this computed field, only used when `by_alias=True`
        alias_priority: priority of the alias. This affects whether an alias generator is used
        title: Title to use when including this computed field in JSON Schema
        description: Description to use when including this computed field in JSON Schema, defaults to the function's
            docstring
        examples: Example values to use when including this computed field in JSON Schema
        json_schema_extra: A dict or callable to provide extra JSON schema properties.
        repr: whether to include this computed field in model repr.
            Default is `False` for private properties and `True` for public properties.
        return_type: optional return for serialization logic to expect when serializing to JSON, if included
            this must be correct, otherwise a `TypeError` is raised.
            If you don't include a return type Any is used, which does runtime introspection to handle arbitrary
            objects.

    Returns:
        A proxy wrapper for the property.
    """

    def dec(f: Any) -> Any:
        nonlocal description, return_type, alias_priority
        unwrapped = _decorators.unwrap_wrapped_function(f)
        if description is None and unwrapped.__doc__:
            description = inspect.cleandoc(unwrapped.__doc__)

        # if the function isn't already decorated with `@property` (or another descriptor), then we wrap it now
        f = _decorators.ensure_property(f)
        alias_priority = (alias_priority or 2) if alias is not None else None

        if repr is None:
            repr_: bool = False if _wrapped_property_is_private(property_=f) else True
        else:
            repr_ = repr

        dec_info = ComputedFieldInfo(
            f, return_type, alias, alias_priority, title, description, examples, json_schema_extra, repr_
        )
        return _decorators.PydanticDescriptorProxy(f, dec_info)

    if __f is None:
        return dec
    else:
        return dec(__f)



from typing import Tuple, Dict

from stigg.generated import FetchEntitlementsQuery, GetPaywallInput, FetchEntitlementQuery, GetActiveSubscriptionsInput


def build_get_entitlement_data(edge_url: str, query: FetchEntitlementQuery) -> Tuple[str, Dict]:
  url = f"{edge_url}/v1/c/{query.customer_id}/entitlements.json"
  params = {}
  if query.resource_id is not None:
    params["resourceId"] = query.resource_id

  if query.feature_id is not None:
    params["featureId"] = query.feature_id

  if query.options is not None and query.options.requested_usage is not None:
    params['requestedUsage'] = query.options.requested_usage

  return url, params


def build_get_entitlements_data(edge_url: str, query: FetchEntitlementsQuery) -> Tuple[str, Dict]:
    url = f"{edge_url}/v1/c/{query.customer_id}/entitlements.json"
    params = {}
    if query.resource_id is not None:
        params["resourceId"] = query.resource_id

    return url, params


def build_get_paywall_data(edge_url: str, _input: GetPaywallInput) -> Tuple[str, Dict]:
    if _input.product_id is not None:
        url = f"{edge_url}/v1/p/{_input.product_id}/paywall.json"
    else:
        url = f"{edge_url}/v1/paywall.json"

    params = {}
    if _input.billing_country_code is not None:
        params["billingCountryCode"] = _input.billing_country_code

    if _input.fetch_all_countries_prices is not None:
        params["fetchAllCountriesPrices"] = _input.fetch_all_countries_prices

    if _input.include_hidden_plans is not None:
        params["includeHiddenPlans"] = _input.include_hidden_plans

    return url, params

def build_get_subscriptions_data(edge_url: str, _input: GetActiveSubscriptionsInput) -> Tuple[str, Dict]:
    url = f"{edge_url}/v1/c/{_input.customer_id}/subscriptions.json"

    params = {}
    if _input.resource_id is not None:
        params["resourceId"] = _input.resource_id

    if _input.resource_ids is not None:
        params["resourceIds"] = _input.resource_ids

    return url, params



# Generated by ariadne-codegen
# Source: ../api-client-schema/src/generated/schema.graphql

from enum import Enum


class AccessDeniedReason(str, Enum):
    BudgetExceeded = "BudgetExceeded"
    CustomerIsArchived = "CustomerIsArchived"
    CustomerNotFound = "CustomerNotFound"
    CustomerResourceNotFound = "CustomerResourceNotFound"
    FeatureNotFound = "FeatureNotFound"
    NoActiveSubscription = "NoActiveSubscription"
    NoFeatureEntitlementInSubscription = "NoFeatureEntitlementInSubscription"
    RequestedUsageExceedingLimit = "RequestedUsageExceedingLimit"
    Unknown = "Unknown"


class AccountAccessMethod(str, Enum):
    AUTHORIZED_DOMAIN = "AUTHORIZED_DOMAIN"
    INVITE_ONLY = "INVITE_ONLY"
    SSO = "SSO"


class AccountAccessRole(str, Enum):
    MEMBER = "MEMBER"
    OWNER = "OWNER"


class AccountStatus(str, Enum):
    ACTIVE = "ACTIVE"
    BLOCKED = "BLOCKED"


class AddonSortFields(str, Enum):
    billingId = "billingId"
    createdAt = "createdAt"
    description = "description"
    displayName = "displayName"
    environmentId = "environmentId"
    id = "id"
    isLatest = "isLatest"
    pricingType = "pricingType"
    productId = "productId"
    refId = "refId"
    status = "status"
    updatedAt = "updatedAt"
    versionNumber = "versionNumber"


class AggregationFunction(str, Enum):
    AVG = "AVG"
    COUNT = "COUNT"
    MAX = "MAX"
    MIN = "MIN"
    SUM = "SUM"
    UNIQUE = "UNIQUE"


class Alignment(str, Enum):
    CENTER = "CENTER"
    LEFT = "LEFT"
    RIGHT = "RIGHT"


class ApiKeySortFields(str, Enum):
    id = "id"


class ApiKeyType(str, Enum):
    CLIENT = "CLIENT"
    SALESFORCE = "SALESFORCE"
    SERVER = "SERVER"
    WORKFLOW = "WORKFLOW"


class Auth0ApplicationType(str, Enum):
    BOTH = "BOTH"
    INDIVIDUAL = "INDIVIDUAL"
    ORGANIZATION = "ORGANIZATION"


class BillingAnchor(str, Enum):
    START_OF_THE_MONTH = "START_OF_THE_MONTH"
    SUBSCRIPTIONS_CONSOLIDATE_BILLING = "SUBSCRIPTIONS_CONSOLIDATE_BILLING"
    SUBSCRIPTION_START = "SUBSCRIPTION_START"


class BillingCadence(str, Enum):
    ONE_OFF = "ONE_OFF"
    RECURRING = "RECURRING"


class BillingModel(str, Enum):
    FLAT_FEE = "FLAT_FEE"
    MINIMUM_SPEND = "MINIMUM_SPEND"
    PER_UNIT = "PER_UNIT"
    USAGE_BASED = "USAGE_BASED"


class BillingPeriod(str, Enum):
    ANNUALLY = "ANNUALLY"
    MONTHLY = "MONTHLY"


class BillingVendorIdentifier(str, Enum):
    STRIPE = "STRIPE"


class ChangeType(str, Enum):
    ADDED = "ADDED"
    DELETED = "DELETED"
    MODIFIED = "MODIFIED"
    REORDERED = "REORDERED"


class ConditionOperation(str, Enum):
    EQUALS = "EQUALS"
    GREATER_THAN = "GREATER_THAN"
    GREATER_THAN_OR_EQUAL = "GREATER_THAN_OR_EQUAL"
    IS_NOT_NULL = "IS_NOT_NULL"
    IS_NULL = "IS_NULL"
    LESS_THAN = "LESS_THAN"
    LESS_THAN_OR_EQUAL = "LESS_THAN_OR_EQUAL"
    NOT_EQUALS = "NOT_EQUALS"


class CouponSortFields(str, Enum):
    billingId = "billingId"
    createdAt = "createdAt"
    description = "description"
    environmentId = "environmentId"
    id = "id"
    name = "name"
    refId = "refId"
    source = "source"
    status = "status"
    type = "type"
    updatedAt = "updatedAt"


class CouponSource(str, Enum):
    STIGG = "STIGG"
    STIGG_ADHOC = "STIGG_ADHOC"
    STRIPE = "STRIPE"


class CouponStatus(str, Enum):
    ACTIVE = "ACTIVE"
    ARCHIVED = "ARCHIVED"


class CouponType(str, Enum):
    FIXED = "FIXED"
    PERCENTAGE = "PERCENTAGE"


class Currency(str, Enum):
    AED = "AED"
    ALL = "ALL"
    AMD = "AMD"
    ANG = "ANG"
    AUD = "AUD"
    AWG = "AWG"
    AZN = "AZN"
    BAM = "BAM"
    BBD = "BBD"
    BDT = "BDT"
    BGN = "BGN"
    BIF = "BIF"
    BMD = "BMD"
    BND = "BND"
    BRL = "BRL"
    BSD = "BSD"
    BWP = "BWP"
    BYN = "BYN"
    BZD = "BZD"
    CAD = "CAD"
    CDF = "CDF"
    CHF = "CHF"
    CLP = "CLP"
    CNY = "CNY"
    CZK = "CZK"
    DJF = "DJF"
    DKK = "DKK"
    DOP = "DOP"
    DZD = "DZD"
    EGP = "EGP"
    ETB = "ETB"
    EUR = "EUR"
    FJD = "FJD"
    GBP = "GBP"
    GEL = "GEL"
    GIP = "GIP"
    GMD = "GMD"
    GNF = "GNF"
    GYD = "GYD"
    HKD = "HKD"
    HRK = "HRK"
    HTG = "HTG"
    IDR = "IDR"
    ILS = "ILS"
    INR = "INR"
    ISK = "ISK"
    JMD = "JMD"
    JPY = "JPY"
    KES = "KES"
    KGS = "KGS"
    KHR = "KHR"
    KMF = "KMF"
    KRW = "KRW"
    KYD = "KYD"
    KZT = "KZT"
    LBP = "LBP"
    LKR = "LKR"
    LRD = "LRD"
    LSL = "LSL"
    MAD = "MAD"
    MDL = "MDL"
    MGA = "MGA"
    MKD = "MKD"
    MMK = "MMK"
    MNT = "MNT"
    MOP = "MOP"
    MRO = "MRO"
    MVR = "MVR"
    MWK = "MWK"
    MXN = "MXN"
    MYR = "MYR"
    MZN = "MZN"
    NAD = "NAD"
    NGN = "NGN"
    NOK = "NOK"
    NPR = "NPR"
    NZD = "NZD"
    PGK = "PGK"
    PHP = "PHP"
    PKR = "PKR"
    PLN = "PLN"
    PYG = "PYG"
    QAR = "QAR"
    RON = "RON"
    RSD = "RSD"
    RUB = "RUB"
    RWF = "RWF"
    SAR = "SAR"
    SBD = "SBD"
    SCR = "SCR"
    SEK = "SEK"
    SGD = "SGD"
    SLE = "SLE"
    SLL = "SLL"
    SOS = "SOS"
    SZL = "SZL"
    THB = "THB"
    TJS = "TJS"
    TOP = "TOP"
    TRY = "TRY"
    TTD = "TTD"
    TZS = "TZS"
    UAH = "UAH"
    UGX = "UGX"
    USD = "USD"
    UZS = "UZS"
    VND = "VND"
    VUV = "VUV"
    WST = "WST"
    XAF = "XAF"
    XCD = "XCD"
    XOF = "XOF"
    XPF = "XPF"
    YER = "YER"
    ZAR = "ZAR"
    ZMW = "ZMW"


class CustomerResourceSortFields(str, Enum):
    createdAt = "createdAt"
    environmentId = "environmentId"
    resourceId = "resourceId"


class CustomerSortFields(str, Enum):
    billingId = "billingId"
    createdAt = "createdAt"
    crmHubspotCompanyId = "crmHubspotCompanyId"
    crmHubspotCompanyUrl = "crmHubspotCompanyUrl"
    crmId = "crmId"
    customerId = "customerId"
    deletedAt = "deletedAt"
    email = "email"
    environmentId = "environmentId"
    id = "id"
    name = "name"
    refId = "refId"
    salesforceId = "salesforceId"
    searchQuery = "searchQuery"
    updatedAt = "updatedAt"


class CustomerSubscriptionSortFields(str, Enum):
    billingCycleAnchor = "billingCycleAnchor"
    billingId = "billingId"
    cancelReason = "cancelReason"
    cancellationDate = "cancellationDate"
    createdAt = "createdAt"
    crmId = "crmId"
    crmLinkUrl = "crmLinkUrl"
    currentBillingPeriodEnd = "currentBillingPeriodEnd"
    currentBillingPeriodStart = "currentBillingPeriodStart"
    customerId = "customerId"
    effectiveEndDate = "effectiveEndDate"
    endDate = "endDate"
    environmentId = "environmentId"
    id = "id"
    oldBillingId = "oldBillingId"
    payingCustomerId = "payingCustomerId"
    paymentCollection = "paymentCollection"
    pricingType = "pricingType"
    refId = "refId"
    resourceId = "resourceId"
    salesforceId = "salesforceId"
    startDate = "startDate"
    status = "status"
    subscriptionId = "subscriptionId"
    trialEndDate = "trialEndDate"


class Department(str, Enum):
    CEO_OR_FOUNDER = "CEO_OR_FOUNDER"
    ENGINEERING = "ENGINEERING"
    GROWTH = "GROWTH"
    MARKETING = "MARKETING"
    MONETIZATION = "MONETIZATION"
    OTHER = "OTHER"
    PRODUCT = "PRODUCT"


class DiscountDurationType(str, Enum):
    FOREVER = "FOREVER"
    ONCE = "ONCE"
    REPEATING = "REPEATING"


class DiscountType(str, Enum):
    FIXED = "FIXED"
    PERCENTAGE = "PERCENTAGE"


class EntitlementBehavior(str, Enum):
    Increment = "Increment"
    Override = "Override"


class EntitlementResetPeriod(str, Enum):
    DAY = "DAY"
    HOUR = "HOUR"
    MONTH = "MONTH"
    WEEK = "WEEK"
    YEAR = "YEAR"


class EntitySelectionMode(str, Enum):
    BLACK_LIST = "BLACK_LIST"
    WHITE_LIST = "WHITE_LIST"


class EnvironmentAccessRole(str, Enum):
    ADMIN = "ADMIN"
    NONE = "NONE"
    SUPPORT = "SUPPORT"
    VIEWER = "VIEWER"


class EnvironmentProvisionStatus(str, Enum):
    DONE = "DONE"
    FAILED = "FAILED"
    IN_PROGRESS = "IN_PROGRESS"
    NOT_PROVISIONED = "NOT_PROVISIONED"


class EnvironmentSortFields(str, Enum):
    createdAt = "createdAt"
    displayName = "displayName"
    id = "id"
    permanentDeletionDate = "permanentDeletionDate"
    slug = "slug"


class EnvironmentType(str, Enum):
    DEVELOPMENT = "DEVELOPMENT"
    PRODUCTION = "PRODUCTION"
    SANDBOX = "SANDBOX"


class ErrorCode(str, Enum):
    AccessDeniedError = "AccessDeniedError"
    AccountNotFoundError = "AccountNotFoundError"
    AddonDependencyMissingError = "AddonDependencyMissingError"
    AddonHasToHavePriceError = "AddonHasToHavePriceError"
    AddonIsCompatibleWithGroup = "AddonIsCompatibleWithGroup"
    AddonIsCompatibleWithPlan = "AddonIsCompatibleWithPlan"
    AddonNotFound = "AddonNotFound"
    AddonQuantityExceedsLimitError = "AddonQuantityExceedsLimitError"
    AddonWithDraftCannotBeDeletedError = "AddonWithDraftCannotBeDeletedError"
    AddonsNotFound = "AddonsNotFound"
    AmountTooLarge = "AmountTooLarge"
    ArchivedCouponCantBeApplied = "ArchivedCouponCantBeApplied"
    AuthCustomerMismatch = "AuthCustomerMismatch"
    AuthCustomerReadonly = "AuthCustomerReadonly"
    AwsMarketplaceIntegrationError = "AwsMarketplaceIntegrationError"
    AwsMarketplaceIntegrationValidationError = (
        "AwsMarketplaceIntegrationValidationError"
    )
    BadUserInput = "BadUserInput"
    BillingIntegrationAlreadyExistsError = "BillingIntegrationAlreadyExistsError"
    BillingIntegrationMissing = "BillingIntegrationMissing"
    BillingPeriodMissingError = "BillingPeriodMissingError"
    CannotAddOverrideEntitlementToPlan = "CannotAddOverrideEntitlementToPlan"
    CannotArchiveFeatureError = "CannotArchiveFeatureError"
    CannotChangeBillingIntegration = "CannotChangeBillingIntegration"
    CannotDeleteCustomerError = "CannotDeleteCustomerError"
    CannotDeleteDefaultIntegration = "CannotDeleteDefaultIntegration"
    CannotDeleteFeatureError = "CannotDeleteFeatureError"
    CannotDeleteProductError = "CannotDeleteProductError"
    CannotEditPackageInNonDraftMode = "CannotEditPackageInNonDraftMode"
    CannotRemovePaymentMethodFromCustomerError = (
        "CannotRemovePaymentMethodFromCustomerError"
    )
    CannotReportUsageForEntitlementWithMeterError = (
        "CannotReportUsageForEntitlementWithMeterError"
    )
    CannotUpdateUnitTransformationError = "CannotUpdateUnitTransformationError"
    CannotUpsertToPackageThatHasDraft = "CannotUpsertToPackageThatHasDraft"
    ChangingPayingCustomerIsNotSupportedError = (
        "ChangingPayingCustomerIsNotSupportedError"
    )
    CheckoutIsNotSupported = "CheckoutIsNotSupported"
    CouponNotFound = "CouponNotFound"
    CustomerAlreadyHaveCustomerCoupon = "CustomerAlreadyHaveCustomerCoupon"
    CustomerAlreadyUsesCoupon = "CustomerAlreadyUsesCoupon"
    CustomerHasNoEmailAddress = "CustomerHasNoEmailAddress"
    CustomerNoBillingId = "CustomerNoBillingId"
    CustomerNotFound = "CustomerNotFound"
    CustomerResourceNotFound = "CustomerResourceNotFound"
    DowngradeBillingPeriodNotSupportedError = "DowngradeBillingPeriodNotSupportedError"
    DraftAddonCantBeArchived = "DraftAddonCantBeArchived"
    DraftPlanCantBeArchived = "DraftPlanCantBeArchived"
    DuplicateAddonProvisionedError = "DuplicateAddonProvisionedError"
    DuplicateIntegrationNotAllowed = "DuplicateIntegrationNotAllowed"
    DuplicateProductValidationError = "DuplicateProductValidationError"
    DuplicatedEntityNotAllowed = "DuplicatedEntityNotAllowed"
    EditAllowedOnDraftPackageOnlyError = "EditAllowedOnDraftPackageOnlyError"
    EntitlementLimitExceededError = "EntitlementLimitExceededError"
    EntitlementUsageOutOfRangeError = "EntitlementUsageOutOfRangeError"
    EntitlementsMustBelongToSamePackage = "EntitlementsMustBelongToSamePackage"
    EntityIdDifferentFromRefIdError = "EntityIdDifferentFromRefIdError"
    EntityIsArchivedError = "EntityIsArchivedError"
    EnvironmentMissing = "EnvironmentMissing"
    ExperimentAlreadyRunning = "ExperimentAlreadyRunning"
    ExperimentNotFoundError = "ExperimentNotFoundError"
    ExperimentStatusError = "ExperimentStatusError"
    FailedToCreateCheckoutSessionError = "FailedToCreateCheckoutSessionError"
    FailedToImportCustomer = "FailedToImportCustomer"
    FailedToImportSubscriptions = "FailedToImportSubscriptions"
    FailedToResolveBillingIntegration = "FailedToResolveBillingIntegration"
    FeatureNotFound = "FeatureNotFound"
    FetchAllCountriesPricesNotAllowed = "FetchAllCountriesPricesNotAllowed"
    FreePlanCantHaveCompatiblePackageGroupError = (
        "FreePlanCantHaveCompatiblePackageGroupError"
    )
    GraphQLAliasesLimitExceeded = "GraphQLAliasesLimitExceeded"
    GraphQLBatchedOperationsLimitExceeded = "GraphQLBatchedOperationsLimitExceeded"
    GraphQLUnsupportedDirective = "GraphQLUnsupportedDirective"
    HubspotIntegrationError = "HubspotIntegrationError"
    IdentityForbidden = "IdentityForbidden"
    ImportAlreadyInProgress = "ImportAlreadyInProgress"
    ImportSubscriptionsBulkError = "ImportSubscriptionsBulkError"
    IncompatibleSubscriptionAddon = "IncompatibleSubscriptionAddon"
    InitStripePaymentMethodError = "InitStripePaymentMethodError"
    IntegrationNotFound = "IntegrationNotFound"
    IntegrationValidationError = "IntegrationValidationError"
    IntegrityViolation = "IntegrityViolation"
    InvalidAddressError = "InvalidAddressError"
    InvalidArgumentError = "InvalidArgumentError"
    InvalidCancellationDate = "InvalidCancellationDate"
    InvalidDoggoSignatureError = "InvalidDoggoSignatureError"
    InvalidEntitlementResetPeriod = "InvalidEntitlementResetPeriod"
    InvalidMemberDelete = "InvalidMemberDelete"
    InvalidMetadataError = "InvalidMetadataError"
    InvalidQuantity = "InvalidQuantity"
    InvalidSubscriptionStatus = "InvalidSubscriptionStatus"
    InvalidUpdatePriceUnitAmountError = "InvalidUpdatePriceUnitAmountError"
    MemberInvitationError = "MemberInvitationError"
    MemberNotFound = "MemberNotFound"
    MergeEnvironmentValidationError = "MergeEnvironmentValidationError"
    MeterMustBeAssociatedToMeteredFeature = "MeterMustBeAssociatedToMeteredFeature"
    MeteringNotAvailableForFeatureType = "MeteringNotAvailableForFeatureType"
    MissingEntityIdError = "MissingEntityIdError"
    MissingSubscriptionInvoiceError = "MissingSubscriptionInvoiceError"
    MultiSubscriptionCantBeAutoCancellationSourceError = (
        "MultiSubscriptionCantBeAutoCancellationSourceError"
    )
    NoFeatureEntitlementError = "NoFeatureEntitlementError"
    NoFeatureEntitlementInSubscription = "NoFeatureEntitlementInSubscription"
    NoProductsAvailable = "NoProductsAvailable"
    OperationNotAllowedDuringInProgressExperiment = (
        "OperationNotAllowedDuringInProgressExperiment"
    )
    PackageAlreadyPublished = "PackageAlreadyPublished"
    PackageGroupMinItemsError = "PackageGroupMinItemsError"
    PackageGroupNotFound = "PackageGroupNotFound"
    PackagePricingTypeNotSet = "PackagePricingTypeNotSet"
    PaymentMethodNotFoundError = "PaymentMethodNotFoundError"
    PlanCannotBePublishWhenBasePlanIsDraft = "PlanCannotBePublishWhenBasePlanIsDraft"
    PlanCannotBePublishWhenCompatibleAddonIsDraft = (
        "PlanCannotBePublishWhenCompatibleAddonIsDraft"
    )
    PlanIsUsedAsDefaultStartPlan = "PlanIsUsedAsDefaultStartPlan"
    PlanIsUsedAsDowngradePlan = "PlanIsUsedAsDowngradePlan"
    PlanNotFound = "PlanNotFound"
    PlanWithChildCantBeDeleted = "PlanWithChildCantBeDeleted"
    PlansCircularDependencyError = "PlansCircularDependencyError"
    PreparePaymentMethodFormError = "PreparePaymentMethodFormError"
    PriceNotFound = "PriceNotFound"
    ProductNotFoundError = "ProductNotFoundError"
    PromotionCodeCustomerNotFirstPurchase = "PromotionCodeCustomerNotFirstPurchase"
    PromotionCodeMaxRedemptionsReached = "PromotionCodeMaxRedemptionsReached"
    PromotionCodeMinimumAmountNotReached = "PromotionCodeMinimumAmountNotReached"
    PromotionCodeNotActive = "PromotionCodeNotActive"
    PromotionCodeNotForCustomer = "PromotionCodeNotForCustomer"
    PromotionCodeNotFound = "PromotionCodeNotFound"
    PromotionalEntitlementNotFoundError = "PromotionalEntitlementNotFoundError"
    RateLimitExceeded = "RateLimitExceeded"
    RecalculateEntitlementsError = "RecalculateEntitlementsError"
    RequiredSsoAuthenticationError = "RequiredSsoAuthenticationError"
    ResyncAlreadyInProgress = "ResyncAlreadyInProgress"
    ScheduledMigrationAlreadyExistsError = "ScheduledMigrationAlreadyExistsError"
    SelectedBillingModelDoesntMatchImportedItemError = (
        "SelectedBillingModelDoesntMatchImportedItemError"
    )
    SingleSubscriptionCantBeAutoCancellationTargetError = (
        "SingleSubscriptionCantBeAutoCancellationTargetError"
    )
    StripeCustomerIsDeleted = "StripeCustomerIsDeleted"
    StripeError = "StripeError"
    SubscriptionAlreadyCanceledOrExpired = "SubscriptionAlreadyCanceledOrExpired"
    SubscriptionAlreadyOnLatestPlanError = "SubscriptionAlreadyOnLatestPlanError"
    SubscriptionDoesNotHaveBillingPeriod = "SubscriptionDoesNotHaveBillingPeriod"
    SubscriptionInvoiceStatusError = "SubscriptionInvoiceStatusError"
    SubscriptionMustHaveSinglePlanError = "SubscriptionMustHaveSinglePlanError"
    SubscriptionNoBillingId = "SubscriptionNoBillingId"
    SubscriptionNotFound = "SubscriptionNotFound"
    TooManySubscriptionsPerCustomer = "TooManySubscriptionsPerCustomer"
    TrialMustBeCancelledImmediately = "TrialMustBeCancelledImmediately"
    UnPublishedPackage = "UnPublishedPackage"
    Unauthenticated = "Unauthenticated"
    UnexpectedError = "UnexpectedError"
    UnsupportedFeatureType = "UnsupportedFeatureType"
    UnsupportedSubscriptionScheduleType = "UnsupportedSubscriptionScheduleType"
    UnsupportedVendorIdentifier = "UnsupportedVendorIdentifier"
    UsageMeasurementDiffOutOfRangeError = "UsageMeasurementDiffOutOfRangeError"
    WorkflowTriggerNotFound = "WorkflowTriggerNotFound"


class EventActor(str, Enum):
    APP_CUSTOMER = "APP_CUSTOMER"
    APP_PUBLIC = "APP_PUBLIC"
    APP_SERVER = "APP_SERVER"
    AWS = "AWS"
    IMPORT = "IMPORT"
    MIGRATION = "MIGRATION"
    SALESFORCE = "SALESFORCE"
    SCHEDULER = "SCHEDULER"
    SERVICE = "SERVICE"
    STRIPE = "STRIPE"
    SUPPORT = "SUPPORT"
    SYSTEM = "SYSTEM"
    USER = "USER"
    WORKFLOW = "WORKFLOW"


class EventEntityType(str, Enum):
    ADDON = "ADDON"
    COUPON = "COUPON"
    CUSTOMER = "CUSTOMER"
    ENTITLEMENT = "ENTITLEMENT"
    FEATURE = "FEATURE"
    IMPORT = "IMPORT"
    MEASUREMENT = "MEASUREMENT"
    PACKAGE = "PACKAGE"
    PACKAGE_GROUP = "PACKAGE_GROUP"
    PLAN = "PLAN"
    PRODUCT = "PRODUCT"
    PROMOTIONAL_ENTITLEMENT = "PROMOTIONAL_ENTITLEMENT"
    SUBSCRIPTION = "SUBSCRIPTION"


class EventLogSortFields(str, Enum):
    createdAt = "createdAt"
    entityId = "entityId"
    environmentId = "environmentId"
    eventLogType = "eventLogType"
    id = "id"
    parentEntityId = "parentEntityId"


class EventLogType(str, Enum):
    ADDON_CREATED = "ADDON_CREATED"
    ADDON_DELETED = "ADDON_DELETED"
    ADDON_UPDATED = "ADDON_UPDATED"
    COUPON_ARCHIVED = "COUPON_ARCHIVED"
    COUPON_CREATED = "COUPON_CREATED"
    COUPON_UPDATED = "COUPON_UPDATED"
    CREATE_SUBSCRIPTION_FAILED = "CREATE_SUBSCRIPTION_FAILED"
    CUSTOMER_CREATED = "CUSTOMER_CREATED"
    CUSTOMER_DELETED = "CUSTOMER_DELETED"
    CUSTOMER_ENTITLEMENT_CALCULATION_TRIGGERED = (
        "CUSTOMER_ENTITLEMENT_CALCULATION_TRIGGERED"
    )
    CUSTOMER_PAYMENT_FAILED = "CUSTOMER_PAYMENT_FAILED"
    CUSTOMER_RESOURCE_ENTITLEMENT_CALCULATION_TRIGGERED = (
        "CUSTOMER_RESOURCE_ENTITLEMENT_CALCULATION_TRIGGERED"
    )
    CUSTOMER_UPDATED = "CUSTOMER_UPDATED"
    EDGE_API_CUSTOMER_DATA_RESYNC = "EDGE_API_CUSTOMER_DATA_RESYNC"
    EDGE_API_DATA_RESYNC = "EDGE_API_DATA_RESYNC"
    EDGE_API_DOGGO_RESYNC = "EDGE_API_DOGGO_RESYNC"
    EDGE_API_PACKAGE_ENTITLEMENTS_DATA_RESYNC = (
        "EDGE_API_PACKAGE_ENTITLEMENTS_DATA_RESYNC"
    )
    EDGE_API_SUBSCRIPTIONS_DATA_RESYNC = "EDGE_API_SUBSCRIPTIONS_DATA_RESYNC"
    ENTITLEMENTS_UPDATED = "ENTITLEMENTS_UPDATED"
    ENTITLEMENT_DENIED = "ENTITLEMENT_DENIED"
    ENTITLEMENT_GRANTED = "ENTITLEMENT_GRANTED"
    ENTITLEMENT_REQUESTED = "ENTITLEMENT_REQUESTED"
    ENTITLEMENT_USAGE_EXCEEDED = "ENTITLEMENT_USAGE_EXCEEDED"
    ENVIRONMENT_DELETED = "ENVIRONMENT_DELETED"
    FEATURE_ARCHIVED = "FEATURE_ARCHIVED"
    FEATURE_CREATED = "FEATURE_CREATED"
    FEATURE_DELETED = "FEATURE_DELETED"
    FEATURE_UPDATED = "FEATURE_UPDATED"
    IMPORT_INTEGRATION_CATALOG_TRIGGERED = "IMPORT_INTEGRATION_CATALOG_TRIGGERED"
    IMPORT_INTEGRATION_CUSTOMERS_TRIGGERED = "IMPORT_INTEGRATION_CUSTOMERS_TRIGGERED"
    IMPORT_SUBSCRIPTIONS_BULK_TRIGGERED = "IMPORT_SUBSCRIPTIONS_BULK_TRIGGERED"
    MEASUREMENT_REPORTED = "MEASUREMENT_REPORTED"
    PACKAGE_GROUP_CREATED = "PACKAGE_GROUP_CREATED"
    PACKAGE_GROUP_UPDATED = "PACKAGE_GROUP_UPDATED"
    PACKAGE_PUBLISHED = "PACKAGE_PUBLISHED"
    PLAN_CREATED = "PLAN_CREATED"
    PLAN_DELETED = "PLAN_DELETED"
    PLAN_UPDATED = "PLAN_UPDATED"
    PRODUCT_CREATED = "PRODUCT_CREATED"
    PRODUCT_DELETED = "PRODUCT_DELETED"
    PRODUCT_UPDATED = "PRODUCT_UPDATED"
    PROMOTIONAL_ENTITLEMENT_EXPIRED = "PROMOTIONAL_ENTITLEMENT_EXPIRED"
    PROMOTIONAL_ENTITLEMENT_GRANTED = "PROMOTIONAL_ENTITLEMENT_GRANTED"
    PROMOTIONAL_ENTITLEMENT_REVOKED = "PROMOTIONAL_ENTITLEMENT_REVOKED"
    PROMOTIONAL_ENTITLEMENT_UPDATED = "PROMOTIONAL_ENTITLEMENT_UPDATED"
    RECALCULATE_ENTITLEMENTS_TRIGGERED = "RECALCULATE_ENTITLEMENTS_TRIGGERED"
    RESYNC_INTEGRATION_TRIGGERED = "RESYNC_INTEGRATION_TRIGGERED"
    SUBSCRIPTIONS_MIGRATED = "SUBSCRIPTIONS_MIGRATED"
    SUBSCRIPTIONS_MIGRATION_TRIGGERED = "SUBSCRIPTIONS_MIGRATION_TRIGGERED"
    SUBSCRIPTION_BILLING_MONTH_ENDS_SOON = "SUBSCRIPTION_BILLING_MONTH_ENDS_SOON"
    SUBSCRIPTION_CANCELED = "SUBSCRIPTION_CANCELED"
    SUBSCRIPTION_CREATED = "SUBSCRIPTION_CREATED"
    SUBSCRIPTION_EXPIRED = "SUBSCRIPTION_EXPIRED"
    SUBSCRIPTION_SPENT_LIMIT_EXCEEDED = "SUBSCRIPTION_SPENT_LIMIT_EXCEEDED"
    SUBSCRIPTION_TRIAL_CONVERTED = "SUBSCRIPTION_TRIAL_CONVERTED"
    SUBSCRIPTION_TRIAL_ENDS_SOON = "SUBSCRIPTION_TRIAL_ENDS_SOON"
    SUBSCRIPTION_TRIAL_EXPIRED = "SUBSCRIPTION_TRIAL_EXPIRED"
    SUBSCRIPTION_TRIAL_STARTED = "SUBSCRIPTION_TRIAL_STARTED"
    SUBSCRIPTION_UPDATED = "SUBSCRIPTION_UPDATED"
    SUBSCRIPTION_USAGE_CHARGE_TRIGGERED = "SUBSCRIPTION_USAGE_CHARGE_TRIGGERED"
    SUBSCRIPTION_USAGE_UPDATED = "SUBSCRIPTION_USAGE_UPDATED"
    SYNC_FAILED = "SYNC_FAILED"
    WIDGET_CONFIGURATION_UPDATED = "WIDGET_CONFIGURATION_UPDATED"


class ExperimentSortFields(str, Enum):
    createdAt = "createdAt"
    environmentId = "environmentId"
    id = "id"
    name = "name"
    productId = "productId"
    refId = "refId"
    status = "status"


class ExperimentStatus(str, Enum):
    COMPLETED = "COMPLETED"
    DRAFT = "DRAFT"
    IN_PROGRESS = "IN_PROGRESS"


class FeatureSortFields(str, Enum):
    createdAt = "createdAt"
    description = "description"
    displayName = "displayName"
    environmentId = "environmentId"
    featureStatus = "featureStatus"
    featureType = "featureType"
    id = "id"
    meterType = "meterType"
    refId = "refId"
    updatedAt = "updatedAt"


class FeatureStatus(str, Enum):
    ACTIVE = "ACTIVE"
    NEW = "NEW"
    SUSPENDED = "SUSPENDED"


class FeatureType(str, Enum):
    BOOLEAN = "BOOLEAN"
    NUMBER = "NUMBER"


class FontWeight(str, Enum):
    BOLD = "BOLD"
    NORMAL = "NORMAL"


class HookSortFields(str, Enum):
    createdAt = "createdAt"
    endpoint = "endpoint"
    environmentId = "environmentId"
    id = "id"
    status = "status"


class HookStatus(str, Enum):
    ACTIVE = "ACTIVE"
    INACTIVE = "INACTIVE"


class ImportIntegrationTaskSortFields(str, Enum):
    createdAt = "createdAt"
    environmentId = "environmentId"
    id = "id"
    status = "status"
    taskType = "taskType"


class IntegrationSortFields(str, Enum):
    createdAt = "createdAt"
    environmentId = "environmentId"
    id = "id"
    vendorIdentifier = "vendorIdentifier"
    vendorType = "vendorType"


class InvoiceLineItemType(str, Enum):
    AddonCharge = "AddonCharge"
    BaseCharge = "BaseCharge"
    InAdvanceCommitmentCharge = "InAdvanceCommitmentCharge"
    MinimumSpendAdjustmentCharge = "MinimumSpendAdjustmentCharge"
    MinimumSpendCharge = "MinimumSpendCharge"
    Other = "Other"
    OverageCharge = "OverageCharge"
    PayAsYouGoCharge = "PayAsYouGoCharge"
    TierCharge = "TierCharge"
    ZeroAmountBaseCharge = "ZeroAmountBaseCharge"


class MemberSortFields(str, Enum):
    createdAt = "createdAt"
    email = "email"
    id = "id"


class MemberStatus(str, Enum):
    INVITED = "INVITED"
    REGISTERED = "REGISTERED"


class MeterType(str, Enum):
    FLUCTUATING = "Fluctuating"
    INCREMENTAL = "Incremental"
    None_ = "None"


class MonthlyAccordingTo(str, Enum):
    StartOfTheMonth = "StartOfTheMonth"
    SubscriptionStart = "SubscriptionStart"


class OverageBillingPeriod(str, Enum):
    MONTHLY = "MONTHLY"
    ON_SUBSCRIPTION_RENEWAL = "ON_SUBSCRIPTION_RENEWAL"


class PackageDTOSortFields(str, Enum):
    billingId = "billingId"
    createdAt = "createdAt"
    description = "description"
    displayName = "displayName"
    environmentId = "environmentId"
    id = "id"
    isLatest = "isLatest"
    pricingType = "pricingType"
    productId = "productId"
    refId = "refId"
    status = "status"
    updatedAt = "updatedAt"
    versionNumber = "versionNumber"


class PackageEntitlementSortFields(str, Enum):
    createdAt = "createdAt"
    environmentId = "environmentId"
    id = "id"
    packageId = "packageId"
    updatedAt = "updatedAt"


class PackageGroupSortFields(str, Enum):
    createdAt = "createdAt"
    displayName = "displayName"
    environmentId = "environmentId"
    isLatest = "isLatest"
    packageGroupId = "packageGroupId"
    productId = "productId"
    status = "status"
    updatedAt = "updatedAt"
    versionNumber = "versionNumber"


class PackageGroupStatus(str, Enum):
    ARCHIVED = "ARCHIVED"
    DRAFT = "DRAFT"
    PUBLISHED = "PUBLISHED"


class PackageStatus(str, Enum):
    ARCHIVED = "ARCHIVED"
    DRAFT = "DRAFT"
    PUBLISHED = "PUBLISHED"


class PaymentCollection(str, Enum):
    ACTION_REQUIRED = "ACTION_REQUIRED"
    FAILED = "FAILED"
    NOT_REQUIRED = "NOT_REQUIRED"
    PROCESSING = "PROCESSING"


class PaymentCollectionMethod(str, Enum):
    CHARGE = "CHARGE"
    INVOICE = "INVOICE"
    NONE = "NONE"


class PaymentMethodType(str, Enum):
    BANK = "BANK"
    CARD = "CARD"
    CASH_APP = "CASH_APP"


class PlanChangeType(str, Enum):
    DOWNGRADE = "DOWNGRADE"
    NONE = "NONE"
    UPGRADE = "UPGRADE"


class PlanSortFields(str, Enum):
    billingId = "billingId"
    createdAt = "createdAt"
    description = "description"
    displayName = "displayName"
    environmentId = "environmentId"
    id = "id"
    isLatest = "isLatest"
    pricingType = "pricingType"
    productId = "productId"
    refId = "refId"
    status = "status"
    updatedAt = "updatedAt"
    versionNumber = "versionNumber"


class PriceSortFields(str, Enum):
    billingCadence = "billingCadence"
    billingId = "billingId"
    billingModel = "billingModel"
    billingPeriod = "billingPeriod"
    createdAt = "createdAt"
    id = "id"
    tiersMode = "tiersMode"


class PricingType(str, Enum):
    CUSTOM = "CUSTOM"
    FREE = "FREE"
    PAID = "PAID"


class ProductSortFields(str, Enum):
    awsMarketplaceProductCode = "awsMarketplaceProductCode"
    awsMarketplaceProductId = "awsMarketplaceProductId"
    createdAt = "createdAt"
    description = "description"
    displayName = "displayName"
    environmentId = "environmentId"
    id = "id"
    isDefaultProduct = "isDefaultProduct"
    multipleSubscriptions = "multipleSubscriptions"
    refId = "refId"
    updatedAt = "updatedAt"


class PromotionalEntitlementPeriod(str, Enum):
    CUSTOM = "CUSTOM"
    LIFETIME = "LIFETIME"
    ONE_MONTH = "ONE_MONTH"
    ONE_WEEK = "ONE_WEEK"
    ONE_YEAR = "ONE_YEAR"
    SIX_MONTH = "SIX_MONTH"


class PromotionalEntitlementSortFields(str, Enum):
    createdAt = "createdAt"
    environmentId = "environmentId"
    id = "id"
    status = "status"
    updatedAt = "updatedAt"


class PromotionalEntitlementStatus(str, Enum):
    Active = "Active"
    Expired = "Expired"
    Paused = "Paused"


class ProrationBehavior(str, Enum):
    CREATE_PRORATIONS = "CREATE_PRORATIONS"
    INVOICE_IMMEDIATELY = "INVOICE_IMMEDIATELY"


class ProvisionSubscriptionStatus(str, Enum):
    PAYMENT_REQUIRED = "PAYMENT_REQUIRED"
    SUCCESS = "SUCCESS"


class PublishMigrationType(str, Enum):
    ALL_CUSTOMERS = "ALL_CUSTOMERS"
    NEW_CUSTOMERS = "NEW_CUSTOMERS"


class ScheduleStrategy(str, Enum):
    END_OF_BILLING_MONTH = "END_OF_BILLING_MONTH"
    END_OF_BILLING_PERIOD = "END_OF_BILLING_PERIOD"
    IMMEDIATE = "IMMEDIATE"


class SortDirection(str, Enum):
    ASC = "ASC"
    DESC = "DESC"


class SortNulls(str, Enum):
    NULLS_FIRST = "NULLS_FIRST"
    NULLS_LAST = "NULLS_LAST"


class SourceType(str, Enum):
    JS_CLIENT_SDK = "JS_CLIENT_SDK"
    NODE_SERVER_SDK = "NODE_SERVER_SDK"
    PERSISTENT_CACHE_SERVICE = "PERSISTENT_CACHE_SERVICE"


class SubscriptionAddonSortFields(str, Enum):
    createdAt = "createdAt"
    id = "id"
    quantity = "quantity"
    updatedAt = "updatedAt"


class SubscriptionCancelReason(str, Enum):
    AutoCancellationRule = "AutoCancellationRule"
    CancelledByBilling = "CancelledByBilling"
    CustomerArchived = "CustomerArchived"
    DetachBilling = "DetachBilling"
    Expired = "Expired"
    Immediate = "Immediate"
    PendingPaymentExpired = "PendingPaymentExpired"
    ScheduledCancellation = "ScheduledCancellation"
    TrialConverted = "TrialConverted"
    TrialEnded = "TrialEnded"
    UpgradeOrDowngrade = "UpgradeOrDowngrade"


class SubscriptionCancellationAction(str, Enum):
    DEFAULT = "DEFAULT"
    REVOKE_ENTITLEMENTS = "REVOKE_ENTITLEMENTS"


class SubscriptionCancellationTime(str, Enum):
    END_OF_BILLING_PERIOD = "END_OF_BILLING_PERIOD"
    IMMEDIATE = "IMMEDIATE"
    SPECIFIC_DATE = "SPECIFIC_DATE"


class SubscriptionCouponStatus(str, Enum):
    ACTIVE = "ACTIVE"
    EXPIRED = "EXPIRED"
    REMOVED = "REMOVED"


class SubscriptionDecisionStrategy(str, Enum):
    PREDEFINED_FREE_PLAN = "PREDEFINED_FREE_PLAN"
    PREDEFINED_TRIAL_PLAN = "PREDEFINED_TRIAL_PLAN"
    REQUESTED_PLAN = "REQUESTED_PLAN"
    SKIPPED_SUBSCRIPTION_CREATION = "SKIPPED_SUBSCRIPTION_CREATION"


class SubscriptionEndSetup(str, Enum):
    CANCEL_SUBSCRIPTION = "CANCEL_SUBSCRIPTION"
    DOWNGRADE_TO_FREE = "DOWNGRADE_TO_FREE"


class SubscriptionEntitlementSortFields(str, Enum):
    createdAt = "createdAt"
    environmentId = "environmentId"
    id = "id"
    subscriptionId = "subscriptionId"
    updatedAt = "updatedAt"


class SubscriptionInvoiceBillingReason(str, Enum):
    BILLING_CYCLE = "BILLING_CYCLE"
    MANUAL = "MANUAL"
    MINIMUM_INVOICE_AMOUNT_EXCEEDED = "MINIMUM_INVOICE_AMOUNT_EXCEEDED"
    OTHER = "OTHER"
    SUBSCRIPTION_CREATION = "SUBSCRIPTION_CREATION"
    SUBSCRIPTION_UPDATE = "SUBSCRIPTION_UPDATE"


class SubscriptionInvoiceStatus(str, Enum):
    CANCELED = "CANCELED"
    OPEN = "OPEN"
    PAID = "PAID"


class SubscriptionMigrationTaskSortFields(str, Enum):
    createdAt = "createdAt"
    environmentId = "environmentId"
    id = "id"
    status = "status"
    taskType = "taskType"


class SubscriptionMigrationTime(str, Enum):
    END_OF_BILLING_PERIOD = "END_OF_BILLING_PERIOD"
    IMMEDIATE = "IMMEDIATE"


class SubscriptionPriceSortFields(str, Enum):
    billingModel = "billingModel"
    createdAt = "createdAt"
    featureId = "featureId"
    hasSoftLimit = "hasSoftLimit"
    id = "id"
    updatedAt = "updatedAt"
    usageLimit = "usageLimit"


class SubscriptionQuerySortFields(str, Enum):
    createdAt = "createdAt"
    customerId = "customerId"
    environmentId = "environmentId"
    productId = "productId"
    resourceId = "resourceId"
    status = "status"


class SubscriptionScheduleStatus(str, Enum):
    Canceled = "Canceled"
    Done = "Done"
    Failed = "Failed"
    PendingPayment = "PendingPayment"
    Scheduled = "Scheduled"


class SubscriptionScheduleType(str, Enum):
    AdditionalMetaData = "AdditionalMetaData"
    Addon = "Addon"
    BillingInfoMetadata = "BillingInfoMetadata"
    BillingPeriod = "BillingPeriod"
    Coupon = "Coupon"
    Downgrade = "Downgrade"
    MigrateToLatest = "MigrateToLatest"
    Plan = "Plan"
    PriceOverride = "PriceOverride"
    UnitAmount = "UnitAmount"


class SubscriptionStartSetup(str, Enum):
    FREE_PLAN = "FREE_PLAN"
    PLAN_SELECTION = "PLAN_SELECTION"
    TRIAL_PERIOD = "TRIAL_PERIOD"


class SubscriptionStatus(str, Enum):
    ACTIVE = "ACTIVE"
    CANCELED = "CANCELED"
    EXPIRED = "EXPIRED"
    IN_TRIAL = "IN_TRIAL"
    NOT_STARTED = "NOT_STARTED"
    PAYMENT_PENDING = "PAYMENT_PENDING"


class SubscriptionUpdateUsageCutoffBehavior(str, Enum):
    ALWAYS_RESET = "ALWAYS_RESET"
    BILLING_PERIOD_CHANGE = "BILLING_PERIOD_CHANGE"
    NEVER_RESET = "NEVER_RESET"


class SyncStatus(str, Enum):
    ERROR = "ERROR"
    NO_SYNC_REQUIRED = "NO_SYNC_REQUIRED"
    PENDING = "PENDING"
    SUCCESS = "SUCCESS"


class TaskStatus(str, Enum):
    CANCELED = "CANCELED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    IN_PROGRESS = "IN_PROGRESS"
    PARTIALLY_FAILED = "PARTIALLY_FAILED"
    PENDING = "PENDING"


class TaskType(str, Enum):
    IMPORT_INTEGRATION_CATALOG = "IMPORT_INTEGRATION_CATALOG"
    IMPORT_INTEGRATION_CUSTOMERS = "IMPORT_INTEGRATION_CUSTOMERS"
    IMPORT_SUBSCRIPTIONS_BULK = "IMPORT_SUBSCRIPTIONS_BULK"
    RECALCULATE_BATCH_ENTITLEMENTS = "RECALCULATE_BATCH_ENTITLEMENTS"
    RECALCULATE_ENTITLEMENTS = "RECALCULATE_ENTITLEMENTS"
    RESYNC_INTEGRATION = "RESYNC_INTEGRATION"
    SUBSCRIPTION_MIGRATION = "SUBSCRIPTION_MIGRATION"
    SUBSCRIPTION_MIGRATION_V2 = "SUBSCRIPTION_MIGRATION_V2"


class TiersMode(str, Enum):
    GRADUATED = "GRADUATED"
    VOLUME = "VOLUME"


class TrialEndBehavior(str, Enum):
    CANCEL_SUBSCRIPTION = "CANCEL_SUBSCRIPTION"
    CONVERT_TO_PAID = "CONVERT_TO_PAID"


class TrialPeriodUnits(str, Enum):
    DAY = "DAY"
    MONTH = "MONTH"


class UnitTransformationRound(str, Enum):
    DOWN = "DOWN"
    UP = "UP"


class UsageMarkerType(str, Enum):
    PERIODIC_RESET = "PERIODIC_RESET"
    SUBSCRIPTION_CHANGE_RESET = "SUBSCRIPTION_CHANGE_RESET"


class UsageMeasurementSortFields(str, Enum):
    createdAt = "createdAt"
    environmentId = "environmentId"
    id = "id"


class UsageUpdateBehavior(str, Enum):
    DELTA = "DELTA"
    SET = "SET"


class VendorIdentifier(str, Enum):
    AUTH0 = "AUTH0"
    AWS_MARKETPLACE = "AWS_MARKETPLACE"
    BIG_QUERY = "BIG_QUERY"
    HUBSPOT = "HUBSPOT"
    OPEN_FGA = "OPEN_FGA"
    SALESFORCE = "SALESFORCE"
    SNOWFLAKE = "SNOWFLAKE"
    STRIPE = "STRIPE"
    ZUORA = "ZUORA"


class VendorType(str, Enum):
    AUTH = "AUTH"
    BILLING = "BILLING"
    CRM = "CRM"
    DATA_EXPORT = "DATA_EXPORT"
    MARKETPLACE = "MARKETPLACE"


class WeeklyAccordingTo(str, Enum):
    EveryFriday = "EveryFriday"
    EveryMonday = "EveryMonday"
    EverySaturday = "EverySaturday"
    EverySunday = "EverySunday"
    EveryThursday = "EveryThursday"
    EveryTuesday = "EveryTuesday"
    EveryWednesday = "EveryWednesday"
    SubscriptionStart = "SubscriptionStart"


class WidgetType(str, Enum):
    CHECKOUT = "CHECKOUT"
    CUSTOMER_PORTAL = "CUSTOMER_PORTAL"
    PAYWALL = "PAYWALL"


class YearlyAccordingTo(str, Enum):
    SubscriptionStart = "SubscriptionStart"


class experimentGroupType(str, Enum):
    CONTROL = "CONTROL"
    VARIANT = "VARIANT"
